@@ -101,7 +101,7 @@ classdef Stage < handle
     end
     function varargout = state(obj,varargin)
       % Create a state.
-      % Arguments: n_rows=1, n_cols=1, quad=False, meta=None
+      % Arguments: n_rows=1, n_cols=1, quad=False, scale=1, meta=None
       %         You must supply a derivative for the state with :obj:`~rockit.stage.Stage.set_der`
       % 
       %         Parameters
@@ -129,7 +129,7 @@ classdef Stage < handle
       %         >>> ocp.set_initial(x, sin(ocp.t)) # Optional: give initial guess
       %         
       global pythoncasadiinterface
-      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','quad','meta'});
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','quad','scale','meta'});
       meta = py.None;
       try
         st = dbstack('-completenames',1);
@@ -147,9 +147,29 @@ classdef Stage < handle
       end
       varargout = pythoncasadiinterface.python2matlab_ret(res);
     end
+    function varargout = register_state(obj,varargin)
+      global pythoncasadiinterface
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'x','quad','scale','meta'});
+      meta = py.None;
+      try
+        st = dbstack('-completenames',1);
+        if length(st)>0
+          meta = struct('stacktrace', {{st(1)}});
+          meta = pythoncasadiinterface.matlab2python(meta);
+        end
+      catch
+      end
+      kwargs = {kwargs{:} 'meta' meta};
+      if isempty(kwargs)
+        res = obj.parent.register_state(args{:});
+      else
+        res = obj.parent.register_state(args{:},pyargs(kwargs{:}));
+      end
+      varargout = pythoncasadiinterface.python2matlab_ret(res);
+    end
     function varargout = algebraic(obj,varargin)
       % Create an algebraic variable
-      % Arguments: n_rows=1, n_cols=1, meta=None
+      % Arguments: n_rows=1, n_cols=1, scale=1, meta=None
       %         You must supply an algebraic relation with:obj:`~rockit.stage.Stage.set_alg`
       % 
       %         Parameters
@@ -167,7 +187,7 @@ classdef Stage < handle
       %             A CasADi symbol representing an algebraic variable
       %         
       global pythoncasadiinterface
-      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','meta'});
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','scale','meta'});
       meta = py.None;
       try
         st = dbstack('-completenames',1);
@@ -185,9 +205,63 @@ classdef Stage < handle
       end
       varargout = pythoncasadiinterface.python2matlab_ret(res);
     end
-    function varargout = variable(obj,varargin)
+    function varargout = register_algebraic(obj,varargin)
       global pythoncasadiinterface
-      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','grid','meta'});
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'z','scale','meta'});
+      meta = py.None;
+      try
+        st = dbstack('-completenames',1);
+        if length(st)>0
+          meta = struct('stacktrace', {{st(1)}});
+          meta = pythoncasadiinterface.matlab2python(meta);
+        end
+      catch
+      end
+      kwargs = {kwargs{:} 'meta' meta};
+      if isempty(kwargs)
+        res = obj.parent.register_algebraic(args{:});
+      else
+        res = obj.parent.register_algebraic(args{:},pyargs(kwargs{:}));
+      end
+      varargout = pythoncasadiinterface.python2matlab_ret(res);
+    end
+    function varargout = variable(obj,varargin)
+      % Create a variable
+      % Arguments: n_rows=1, n_cols=1, grid=, scale=1, include_last=False, meta=None
+      % 
+      %         Variables are unknowns in the Optimal Control problem
+      %         for which we seek optimal values.
+      % 
+      %         Parameters
+      %         ----------
+      %         n_rows : int, optional
+      %             Number of rows
+      %         n_cols : int, optional
+      %             Number of columns
+      %         grid : string, optional
+      %             Default is '', resulting in a single variable available
+      %             over the whole optimal control horizon.
+      %             For MultipleShooting, 'control' can be used to
+      %             declare a variable that is unique to every control interval.
+      %             include_last determines if a unique entry is foreseen at the tf edge.
+      % 
+      % 
+      %         Returns
+      %         -------
+      %         s : :obj:`~casadi.MX`
+      %             A CasADi symbol representing a variable
+      % 
+      %         Examples
+      %         --------
+      % 
+      %         >>> ocp = Ocp()
+      %         >>> v = ocp.variable()
+      %         >>> x = ocp.state()
+      %         >>> ocp.set_der(x, v)
+      %         >>> ocp.set_initial(v, 3)
+      %         
+      global pythoncasadiinterface
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','grid','scale','include_last','meta'});
       meta = py.None;
       try
         st = dbstack('-completenames',1);
@@ -205,13 +279,68 @@ classdef Stage < handle
       end
       varargout = pythoncasadiinterface.python2matlab_ret(res);
     end
+    function varargout = register_variable(obj,varargin)
+      global pythoncasadiinterface
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'v','grid','scale','include_last','meta'});
+      meta = py.None;
+      try
+        st = dbstack('-completenames',1);
+        if length(st)>0
+          meta = struct('stacktrace', {{st(1)}});
+          meta = pythoncasadiinterface.matlab2python(meta);
+        end
+      catch
+      end
+      kwargs = {kwargs{:} 'meta' meta};
+      if isempty(kwargs)
+        res = obj.parent.register_variable(args{:});
+      else
+        res = obj.parent.register_variable(args{:},pyargs(kwargs{:}));
+      end
+      varargout = pythoncasadiinterface.python2matlab_ret(res);
+    end
     function varargout = parameter(obj,varargin)
+      % Create a parameter
+      % Arguments: n_rows=1, n_cols=1, grid=, scale=1, include_last=False, meta=None
       % 
-      % Arguments: n_rows=1, n_cols=1, grid=, meta=None
-      %         Create a parameter
+      %         Parameters are symbols of an Optimal COntrol problem
+      %         that are externally imposed, but not hardcoded.
+      % 
+      %         The advantage of parameters over simple numbers/numerical matrices comes
+      %         when you need to solve multiple different Optimal Control problems.
+      %         Parameters avoid the need to initialize new problems form scratch all the time;
+      %         the problem becomes parametric.
+      % 
+      % 
+      %         Parameters
+      %         ----------
+      %         n_rows : int, optional
+      %             Number of rows
+      %         n_cols : int, optional
+      %             Number of columns
+      %         grid : string, optional
+      %             Default is '', resulting in a single parameter available
+      %             over the whole optimal control horizon. 
+      %             For MultipleShooting, 'control' can be used to
+      %             declare a parameter that is unique to every control interval.
+      %             include_last determines if a unique entry is foreseen at the tf edge.
+      % 
+      %         Returns
+      %         -------
+      %         s : :obj:`~casadi.MX`
+      %             A CasADi symbol representing a parameter
+      % 
+      %         Examples
+      %         --------
+      % 
+      %         >>> ocp = Ocp()
+      %         >>> p = ocp.parameter()
+      %         >>> x = ocp.state()
+      %         >>> ocp.set_der(x, p)
+      %         >>> ocp.set_value(p, 3)
       %         
       global pythoncasadiinterface
-      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','grid','meta'});
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','grid','scale','include_last','meta'});
       meta = py.None;
       try
         st = dbstack('-completenames',1);
@@ -229,9 +358,29 @@ classdef Stage < handle
       end
       varargout = pythoncasadiinterface.python2matlab_ret(res);
     end
+    function varargout = register_parameter(obj,varargin)
+      global pythoncasadiinterface
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'p','grid','scale','include_last','meta'});
+      meta = py.None;
+      try
+        st = dbstack('-completenames',1);
+        if length(st)>0
+          meta = struct('stacktrace', {{st(1)}});
+          meta = pythoncasadiinterface.matlab2python(meta);
+        end
+      catch
+      end
+      kwargs = {kwargs{:} 'meta' meta};
+      if isempty(kwargs)
+        res = obj.parent.register_parameter(args{:});
+      else
+        res = obj.parent.register_parameter(args{:},pyargs(kwargs{:}));
+      end
+      varargout = pythoncasadiinterface.python2matlab_ret(res);
+    end
     function varargout = control(obj,varargin)
       % Create a control signal to optimize for
-      % Arguments: n_rows=1, n_cols=1, order=0, meta=None
+      % Arguments: n_rows=1, n_cols=1, order=0, scale=1, meta=None
       % 
       %         A control signal is parametrized as a piecewise polynomial.
       %         By default (order=0), it is piecewise constant.
@@ -259,7 +408,7 @@ classdef Stage < handle
       %         >>> ocp.set_initial(u, sin(ocp.t)) # Optional: give initial guess
       %         
       global pythoncasadiinterface
-      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','order','meta'});
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,0,{'n_rows','n_cols','order','scale','meta'});
       meta = py.None;
       try
         st = dbstack('-completenames',1);
@@ -277,7 +426,49 @@ classdef Stage < handle
       end
       varargout = pythoncasadiinterface.python2matlab_ret(res);
     end
+    function varargout = register_control(obj,varargin)
+      global pythoncasadiinterface
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'u','scale','meta'});
+      meta = py.None;
+      try
+        st = dbstack('-completenames',1);
+        if length(st)>0
+          meta = struct('stacktrace', {{st(1)}});
+          meta = pythoncasadiinterface.matlab2python(meta);
+        end
+      catch
+      end
+      kwargs = {kwargs{:} 'meta' meta};
+      if isempty(kwargs)
+        res = obj.parent.register_control(args{:});
+      else
+        res = obj.parent.register_control(args{:},pyargs(kwargs{:}));
+      end
+      varargout = pythoncasadiinterface.python2matlab_ret(res);
+    end
     function varargout = set_value(obj,varargin)
+      % Set a value for a parameter
+      % Arguments: parameter, value
+      % 
+      %         All variables must be given a value before an optimal control problem can be solved.
+      % 
+      %         Parameters
+      %         ----------
+      %         parameter : :obj:`~casadi.MX`
+      %             The parameter symbol to initialize
+      %         value : number
+      %             The value
+      % 
+      % 
+      %         Examples
+      %         --------
+      % 
+      %         >>> ocp = Ocp()
+      %         >>> p = ocp.parameter()
+      %         >>> x = ocp.state()
+      %         >>> ocp.set_der(x, p)
+      %         >>> ocp.set_value(p, 3)
+      %         
       global pythoncasadiinterface
       [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,2,{'parameter','value'});
       if isempty(kwargs)
@@ -288,6 +479,38 @@ classdef Stage < handle
       varargout = pythoncasadiinterface.python2matlab_ret(res);
     end
     function varargout = set_initial(obj,varargin)
+      % Provide an initial guess
+      % Arguments: var, value, priority=True
+      % 
+      %         Many Optimal Control solution methods are based on
+      %         iterative numerical recipes.
+      %         The initial guess, or starting point, may influence the
+      %         convergence behavior and the quality of the solution.
+      % 
+      %         By default, all states, controls, and variables are initialized with zero.
+      %         Use set_initial to provide a non-zero initial guess.
+      % 
+      %         Parameters
+      %         ----------
+      %         var : :obj:`~casadi.MX`
+      %             The variable, state or control symbol (shape n-by-1) to initialize
+      %         value : :obj:`~casadi.MX`
+      %             The value to initialize with. Possibilities:
+      %               * scalar number (repeated to fit the shape of `var` if needed)
+      %               * numeric matrix of shape n-by-N or n-by-(N+1) in the case of MultipleShooting
+      %               * CasADi symbolic expression dependent on ocp.t 
+      % 
+      %         Examples
+      %         --------
+      % 
+      %         >>> ocp = Ocp()
+      %         >>> x = ocp.state()
+      %         >>> u = ocp.control()
+      %         >>> ocp.set_der(x, u)
+      %         >>> ocp.set_initial(u, 1)
+      %         >>> ocp.set_initial(u, linspace(0,1,10))
+      %         >>> ocp.set_initial(u, sin(ocp.t))
+      %         
       global pythoncasadiinterface
       [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,2,{'var','value','priority'});
       if isempty(kwargs)
@@ -299,7 +522,7 @@ classdef Stage < handle
     end
     function varargout = set_der(obj,varargin)
       % Assign a right-hand side to a state derivative
-      % Arguments: state, der
+      % Arguments: state, der, scale=1
       % 
       %         Parameters
       %         ----------
@@ -319,7 +542,7 @@ classdef Stage < handle
       %         >>> ocp.set_der(x, -x)
       %         
       global pythoncasadiinterface
-      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,2,{'state','der'});
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,2,{'state','der','scale'});
       if isempty(kwargs)
         res = obj.parent.set_der(args{:});
       else
@@ -358,7 +581,7 @@ classdef Stage < handle
     end
     function varargout = add_alg(obj,varargin)
       global pythoncasadiinterface
-      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'constr'});
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'constr','scale'});
       if isempty(kwargs)
         res = obj.parent.add_alg(args{:});
       else
@@ -402,7 +625,7 @@ classdef Stage < handle
     end
     function varargout = sum(obj,varargin)
       % Compute a sum
-      % Arguments: expr, grid=inf
+      % Arguments: expr, grid=control, include_last=False
       % 
       %         Parameters
       %         ----------
@@ -410,12 +633,11 @@ classdef Stage < handle
       %             An expression to integrate over the state time domain (from t0 to tf=t0+T)
       %         grid : str
       %             Possible entries:
-      %                 inf: the integral is performed using the integrator defined for the stage
       %                 control: the integral is evaluated as a sum on the control grid (start of each control interval)
       %                          Note that the final state is not included in this definition
       %         
       global pythoncasadiinterface
-      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'expr','grid'});
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'expr','grid','include_last'});
       if isempty(kwargs)
         res = obj.parent.sum(args{:});
       else
@@ -520,7 +742,7 @@ classdef Stage < handle
     end
     function varargout = subject_to(obj,varargin)
       % Adds a constraint to the problem
-      % Arguments: constr, grid=None, include_first=True, include_last=True, meta=None
+      % Arguments: constr, grid=None, include_first=True, include_last=True, scale=1, refine=1, group_refine=<rockit.grouping_techniques.GroupingTechnique object at 0x7fc632052590>, group_dim=<rockit.grouping_techniques.GroupingTechnique object at 0x7fc6320525d0>, group_control=<rockit.grouping_techniques.GroupingTechnique object at 0x7fc632052610>, meta=None
       % 
       %         Parameters
       %         ----------
@@ -540,7 +762,20 @@ classdef Stage < handle
       %                 inf: use mathematical guarantee for the whole control interval (only possible for polynomials of states and controls)
       %                 integrator: constrain at integrator edges
       %                 integrator_roots: constrain at integrator roots (e.g. collocation points excluding 0)
-      % 
+      %         include_first : bool
+      %             Enforce constraint also at t0
+      %         include_last : bool or "auto"
+      %             Enforce constraint also at tf
+      %             "auto" mode will only enforce the constraint if it is not dependent on a control signal,
+      %             since typically control signals are not defined at tf.
+      %         refine : int, optional
+      %             Refine grid used in constraining by a certain factor with respect to the control grid
+      %         group_refine : GroupTechnique, optional
+      %             Group constraints together along the refine axis
+      %         group_dim : GroupTechnique, optional
+      %             Group vector-valued constraints along the vector dimension into a scalar constraint
+      %         group_control : GroupTechnique, optional
+      %             Group constraints together along the control grid
       %         Examples
       %         --------
       % 
@@ -552,7 +787,7 @@ classdef Stage < handle
       %         >>> ocp.subject_to( ocp.at_tf(x) == 0)  # boundary constraint
       %         
       global pythoncasadiinterface
-      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'constr','grid','include_first','include_last','meta'});
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'constr','grid','include_first','include_last','scale','refine','group_refine','group_dim','group_control','meta'});
       meta = py.None;
       try
         st = dbstack('-completenames',1);
@@ -707,6 +942,24 @@ classdef Stage < handle
       end
       varargout = pythoncasadiinterface.python2matlab_ret(res);
     end
+    function varargout = is_parametric(obj,varargin)
+      % Does the expression depend only on parameters?
+      % Arguments: expr
+      % 
+      %         Returns
+      %         -------
+      %         res : bool
+      % 
+      %         
+      global pythoncasadiinterface
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'expr'});
+      if isempty(kwargs)
+        res = obj.parent.is_parametric(args{:});
+      else
+        res = obj.parent.is_parametric(args{:},pyargs(kwargs{:}));
+      end
+      varargout = pythoncasadiinterface.python2matlab_ret(res);
+    end
     function varargout = clone(obj,varargin)
       global pythoncasadiinterface
       [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'parent','kwargs'});
@@ -738,7 +991,7 @@ classdef Stage < handle
       %         grid : `str`
       %             At which points in time to sample, options are
       %             'control' or 'integrator' (at integrator discretization
-      %             level) or 'integrator_roots'.
+      %             level), 'integrator_roots', 'gist'.            
       %         refine : int, optional
       %             Refine grid by evaluation the polynomal of the integrater at
       %             intermediate points ("refine" points per interval).
@@ -784,6 +1037,24 @@ classdef Stage < handle
       end
       varargout = pythoncasadiinterface.python2matlab_ret(res);
     end
+    function varargout = initial_value(obj,varargin)
+      % Get the value of an expression at initial guess
+      % Arguments: expr
+      % 
+      %         Parameters
+      %         ----------
+      %         expr : :obj:`casadi.MX`
+      %             Arbitrary expression containing no signals (states, controls) ...
+      %         
+      global pythoncasadiinterface
+      [args,kwargs] = pythoncasadiinterface.matlab2python_arg(varargin,1,{'expr'});
+      if isempty(kwargs)
+        res = obj.parent.initial_value(args{:});
+      else
+        res = obj.parent.initial_value(args{:},pyargs(kwargs{:}));
+      end
+      varargout = pythoncasadiinterface.python2matlab_ret(res);
+    end
     function varargout = discrete_system(obj,varargin)
       % Hack
       % Arguments: 
